@@ -36,12 +36,12 @@
 """2D shapes.
 
 This module provides classes for a variety of simplistic 2D shapes,
-such as Rectangles, Circles, and Lines. These shapes are are made
+such as Rectangles, Circles, and Lines. These shapes are made
 internally from OpenGL primitives, and provide excellent performance
 when drawn as part of a :py:class:`~pyglet.graphics.Batch`.
 Convenience methods are provided for positioning, changing color
-and opacity, and rotation (where applicible). To create more
-complex shapes than what is provided here, the lower evel
+and opacity, and rotation (where applicable). To create more
+complex shapes than what is provided here, the lower level
 graphics API is more appropriate.
 See the :ref:`guide_graphics` for more details.
 
@@ -60,6 +60,7 @@ A simple example of drawing shapes::
     rectangle.rotation = 33
     line = shapes.Line(100, 100, 100, 200, width=19, batch=batch)
     line2 = shapes.Line(150, 150, 444, 111, width=4, color=(200, 20, 20), batch=batch)
+    star = shapes.Star(800, 400, 60, 40, num_spikes=20, color=(255, 255, 0), batch=batch)
 
     @window.event
     def on_draw():
@@ -142,11 +143,8 @@ class _ShapeBase:
     _vertex_list = None
 
     def __del__(self):
-        try:
-            if self._vertex_list is not None:
-                self._vertex_list.delete()
-        except:
-            pass
+        if self._vertex_list is not None:
+            self._vertex_list.delete()
 
     def _update_position(self):
         raise NotImplementedError
@@ -322,7 +320,7 @@ class Arc(_ShapeBase):
             `radius` : float
                 The desired radius.
             `segments` : int
-                You can optionally specifify how many distict line segments
+                You can optionally specify how many distinct line segments
                 the arc should be made from. If not specified it will be
                 automatically calculated using the formula:
                 `max(14, int(radius / 1.25))`.
@@ -371,7 +369,7 @@ class Arc(_ShapeBase):
             tau_segs = self._angle / self._segments
             start_angle = self._start_angle - math.radians(self._rotation)
 
-            # Calcuate the outer points of the arc:
+            # Calculate the outer points of the arc:
             points = [(x + (r * math.cos((i * tau_segs) + start_angle)),
                        y + (r * math.sin((i * tau_segs) + start_angle))) for i in range(self._segments + 1)]
 
@@ -429,7 +427,7 @@ class Circle(_ShapeBase):
             `radius` : float
                 The desired radius.
             `segments` : int
-                You can optionally specifify how many distict triangles
+                You can optionally specify how many distinct triangles
                 the circle should be made from. If not specified it will
                 be automatically calculated based using the formula:
                 `max(14, int(radius / 1.25))`.
@@ -463,11 +461,11 @@ class Circle(_ShapeBase):
             r = self._radius
             tau_segs = math.pi * 2 / self._segments
 
-            # Calcuate the outer points of the circle:
+            # Calculate the outer points of the circle:
             points = [(x + (r * math.cos(i * tau_segs)),
                        y + (r * math.sin(i * tau_segs))) for i in range(self._segments)]
 
-            # Create a list of trianges from the points:
+            # Create a list of triangles from the points:
             vertices = []
             for i, point in enumerate(points):
                 triangle = x, y, *points[i - 1], *point
@@ -492,6 +490,110 @@ class Circle(_ShapeBase):
         self._update_position()
 
 
+class Sector(_ShapeBase):
+    def __init__(self, x, y, radius, segments=None, angle=math.tau, start_angle=0,
+                 color=(255, 255, 255), batch=None, group=None):
+        """Create a sector of a circle.
+
+                The sector's anchor point (x, y) defaults to the center of the circle.
+
+                :Parameters:
+                    `x` : float
+                        X coordinate of the sector.
+                    `y` : float
+                        Y coordinate of the sector.
+                    `radius` : float
+                        The desired radius.
+                    `segments` : int
+                        You can optionally specify how many distinct triangles
+                        the sector should be made from. If not specified it will
+                        be automatically calculated based using the formula:
+                        `max(14, int(radius / 1.25))`.
+                    `angle` : float
+                        The angle of the sector, in radians. Defaults to tau (pi * 2),
+                        which is a full circle.
+                    `start_angle` : float
+                        The start angle of the sector, in radians. Defaults to 0.
+                    `color` : (int, int, int)
+                        The RGB color of the sector, specified as a tuple of
+                        three ints in the range of 0-255.
+                    `batch` : `~pyglet.graphics.Batch`
+                        Optional batch to add the sector to.
+                    `group` : `~pyglet.graphics.Group`
+                        Optional parent group of the sector.
+                """
+        self._x = x
+        self._y = y
+        self._radius = radius
+        self._segments = segments or max(14, int(radius / 1.25))
+
+        self._rgb = color
+        self._angle = angle
+        self._start_angle = start_angle
+        self._rotation = 0
+
+        self._batch = batch or Batch()
+        self._group = _ShapeGroup(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, group)
+
+        self._vertex_list = self._batch.add(self._segments * 3, GL_TRIANGLES, self._group, 'v2f', 'c4B')
+        self._update_position()
+        self._update_color()
+
+    def _update_position(self):
+        if not self._visible:
+            vertices = (0,) * self._segments * 6
+        else:
+            x = self._x + self._anchor_x
+            y = self._y + self._anchor_y
+            r = self._radius
+            tau_segs = self._angle / self._segments
+            start_angle = self._start_angle - math.radians(self._rotation)
+
+            # Calculate the outer points of the sector.
+            points = [(x + (r * math.cos((i * tau_segs) + start_angle)),
+                       y + (r * math.sin((i * tau_segs) + start_angle))) for i in range(self._segments + 1)]
+
+            # Create a list of triangles from the points
+            vertices = []
+            for i, point in enumerate(points[1:], start=1):
+                triangle = x, y, *points[i - 1], *point
+                vertices.extend(triangle)
+
+        self._vertex_list.vertices[:] = vertices
+
+    def _update_color(self):
+        self._vertex_list.colors[:] = [*self._rgb, int(self._opacity)] * self._segments * 3
+
+    @property
+    def radius(self):
+        """The radius of the circle.
+
+        :type: float
+        """
+        return self._radius
+
+    @radius.setter
+    def radius(self, value):
+        self._radius = value
+        self._update_position()
+
+    @property
+    def rotation(self):
+        """Clockwise rotation of the sector, in degrees.
+
+        The sector will be rotated about its (anchor_x, anchor_y)
+        position.
+
+        :type: float
+        """
+        return self._rotation
+
+    @rotation.setter
+    def rotation(self, rotation):
+        self._rotation = rotation
+        self._update_position()
+
+
 class Line(_ShapeBase):
     def __init__(self, x, y, x2, y2, width=1, color=(255, 255, 255), batch=None, group=None):
         """Create a line.
@@ -612,7 +714,7 @@ class Rectangle(_ShapeBase):
     def __init__(self, x, y, width, height, color=(255, 255, 255), batch=None, group=None):
         """Create a rectangle or square.
 
-        The rectangles's anchor point defaults to the (x, y) coordinates,
+        The rectangle's anchor point defaults to the (x, y) coordinates,
         which are at the bottom left.
 
         :Parameters:
@@ -726,7 +828,7 @@ class BorderedRectangle(_ShapeBase):
                  border_color=(100, 100, 100), batch=None, group=None):
         """Create a rectangle or square.
 
-        The rectangles's anchor point defaults to the (x, y) coordinates,
+        The rectangle's anchor point defaults to the (x, y) coordinates,
         which are at the bottom left.
 
         :Parameters:
@@ -738,9 +840,14 @@ class BorderedRectangle(_ShapeBase):
                 The width of the rectangle.
             `height` : float
                 The height of the rectangle.
+            `border` : float
+                The thickness of the border.
             `color` : (int, int, int)
                 The RGB color of the rectangle, specified as
                 a tuple of three ints in the range of 0-255.
+            `border_color` : (int, int, int)
+                The RGB color of the rectangle's border, specified as
+                a tuple of three ints in the range of 0-255.
             `batch` : `~pyglet.graphics.Batch`
                 Optional batch to add the rectangle to.
             `group` : `~pyglet.graphics.Group`
@@ -750,6 +857,7 @@ class BorderedRectangle(_ShapeBase):
         self._y = y
         self._width = width
         self._height = height
+        self._rotation = 0
         self._border = border
         self._rgb = color
         self._brgb = border_color
@@ -764,6 +872,44 @@ class BorderedRectangle(_ShapeBase):
     def _update_position(self):
         if not self._visible:
             self._vertex_list.vertices = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
+        elif self._rotation:
+            b = self._border
+            x = self._x
+            y = self._y
+
+            bx1 = -self._anchor_x
+            by1 = -self._anchor_y
+            bx2 = bx1 + self._width
+            by2 = by1 + self._height
+            ix1 = bx1 + b
+            iy1 = by1 + b
+            ix2 = bx2 - b
+            iy2 = by2 - b
+
+            r = -math.radians(self._rotation)
+            cr = math.cos(r)
+            sr = math.sin(r)
+
+            bax = bx1 * cr - by1 * sr + x
+            bay = bx1 * sr + by1 * cr + y
+            bbx = bx2 * cr - by1 * sr + x
+            bby = bx2 * sr + by1 * cr + y
+            bcx = bx2 * cr - by2 * sr + x
+            bcy = bx2 * sr + by2 * cr + y
+            bdx = bx1 * cr - by2 * sr + x
+            bdy = bx1 * sr + by2 * cr + y
+
+            iax = ix1 * cr - iy1 * sr + x
+            iay = ix1 * sr + iy1 * cr + y
+            ibx = ix2 * cr - iy1 * sr + x
+            iby = ix2 * sr + iy1 * cr + y
+            icx = ix2 * cr - iy2 * sr + x
+            icy = ix2 * sr + iy2 * cr + y
+            idx = ix1 * cr - iy2 * sr + x
+            idy = ix1 * sr + iy2 * cr + y
+
+            self._vertex_list.vertices[:] = (iax, iay, ibx, iby, icx, icy, idx, idy,
+                                             bax, bay, bbx, bby, bcx, bcy, bdx, bdy,)
         else:
             b = self._border
             bx1 = self._x - self._anchor_x
@@ -807,6 +953,40 @@ class BorderedRectangle(_ShapeBase):
         self._height = value
         self._update_position()
 
+    @property
+    def rotation(self):
+        """Clockwise rotation of the rectangle, in degrees.
+
+        The Rectangle will be rotated about its (anchor_x, anchor_y)
+        position.
+
+        :type: float
+        """
+        return self._rotation
+
+    @rotation.setter
+    def rotation(self, value):
+        self._rotation = value
+        self._update_position()
+
+    @property
+    def border_color(self):
+        """The rectangle's border color.
+
+        This property sets the color of the border of a bordered rectangle.
+
+        The color is specified as an RGB tuple of integers '(red, green, blue)'.
+        Each color component must be in the range 0 (dark) to 255 (saturated).
+
+        :type: (int, int, int)
+        """
+        return self._brgb
+
+    @border_color.setter
+    def border_color(self, values):
+        self._brgb = list(map(int, values))
+        self._update_color()
+
 
 class Triangle(_ShapeBase):
     def __init__(self, x, y, x2, y2, x3, y3, color=(255, 255, 255), batch=None, group=None):
@@ -946,4 +1126,262 @@ class Triangle(_ShapeBase):
         self._update_position()
 
 
-__all__ = ('Arc', 'Circle', 'Line', 'Rectangle', 'BorderedRectangle', 'Triangle')
+class Star(_ShapeBase):
+    def __init__(self, x, y, outer_radius, inner_radius, num_spikes, rotation=0,
+                 color=(255, 255, 255), batch=None, group=None) -> None:
+        """Create a star.
+
+        The star's anchor point (x, y) defaults to the center of the star.
+
+        :Parameters:
+            `x` : float
+                The X coordinate of the star.
+            `y` : float
+                The Y coordinate of the star.
+            `outer_radius` : float
+                The desired outer radius of the star.
+            `inner_radius` : float
+                The desired inner radius of the star.
+            `num_spikes` : float
+                The desired number of spikes of the star.
+            `rotation` : float
+                The rotation of the star in degrees. A rotation of 0 degrees
+                will result in one spike lining up with the X axis in 
+                positive direction. 
+            `color` : (int, int, int)
+                The RGB color of the star, specified as
+                a tuple of three ints in the range of 0-255.
+            `batch` : `~pyglet.graphics.Batch`
+                Optional batch to add the star to.
+            `group` : `~pyglet.graphics.Group`
+                Optional parent group of the star.
+        """
+        self._x = x
+        self._y = y
+        self._outer_radius = outer_radius
+        self._inner_radius = inner_radius
+        self._num_spikes = num_spikes
+        self._rgb = color
+        self._rotation = rotation
+
+        self._batch = batch or Batch()
+        self._group = _ShapeGroup(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, group)
+
+        self._vertex_list = self._batch.add(self._num_spikes*6, GL_TRIANGLES,
+                                            self._group, 'v2f', 'c4B')
+        self._update_position()
+        self._update_color()
+
+    def _update_position(self):
+        if not self._visible:
+            vertices = (0, 0) * self._num_spikes * 6
+        else:
+            x = self._x + self._anchor_x
+            y = self._y + self._anchor_y
+            r_i = self._inner_radius
+            r_o = self._outer_radius
+
+            # get angle covered by each line (= half a spike)
+            d_theta = math.pi / self._num_spikes
+
+            # phase shift rotation
+            phi = self._rotation / 180 * math.pi
+
+            # calculate alternating points on outer and outer circles
+            points = []
+            for i in range(self._num_spikes):
+                points.append((x + (r_o * math.cos(2*i * d_theta + phi)),
+                               y + (r_o * math.sin(2*i * d_theta + phi))))
+                points.append((x + (r_i * math.cos((2*i+1) * d_theta + phi)),
+                               y + (r_i * math.sin((2*i+1) * d_theta + phi))))
+
+            # create a list of doubled-up points from the points
+            vertices = []
+            for i, point in enumerate(points):
+                triangle = x, y, *points[i - 1], *point
+                vertices.extend(triangle)
+
+        self._vertex_list.vertices[:] = vertices
+
+    def _update_color(self):
+        self._vertex_list.colors[:] = [*self._rgb, int(self._opacity)] * self._num_spikes * 6
+
+    @property
+    def outer_radius(self):
+        """The outer radius of the star."""
+        return self._outer_radius
+
+    @outer_radius.setter
+    def outer_radius(self, value):
+        self._outer_radius = value
+        self._update_position()
+
+    @property
+    def inner_radius(self):
+        """The inner radius of the star."""
+        return self._inner_radius
+
+    @inner_radius.setter
+    def inner_radius(self, value):
+        self._inner_radius = value
+        self._update_position()
+
+    @property
+    def num_spikes(self):
+        """Number of spikes of the star."""
+        return self._num_spikes
+
+    @num_spikes.setter
+    def num_spikes(self, value):
+        self._num_spikes = value
+        self._update_position()
+
+    @property
+    def rotation(self):
+        """Rotation of the star, in degrees.
+        """
+        return self._rotation
+
+    @rotation.setter
+    def rotation(self, rotation):
+        self._rotation = rotation
+        self._update_position()
+
+
+class Polygon(_ShapeBase):
+    def __init__(self, *coordinates, color=(255, 255, 255), batch=None, group=None):
+        """Create a convex polygon.
+
+        The polygon's anchor point defaults to the first vertex point.
+
+        :Parameters:
+            `coordinates` : List[[int, int]]
+                The coordinates for each point in the polygon.
+            `color` : (int, int, int)
+                The RGB color of the polygon, specified as
+                a tuple of three ints in the range of 0-255.
+            `batch` : `~pyglet.graphics.Batch`
+                Optional batch to add the polygon to.
+            `group` : `~pyglet.graphics.Group`
+                Optional parent group of the polygon.
+        """
+
+        # len(self._coordinates) = the number of vertices and sides in the shape.
+        self._coordinates = list(coordinates)
+
+        self._rotation = 0
+
+        self._rgb = color
+
+        self._batch = batch or Batch()
+        self._group = _ShapeGroup(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, group)
+        self._vertex_list = self._batch.add((len(self._coordinates) - 2) * 3, GL_TRIANGLES, self._group, 'v2f', 'c4B')
+        self._update_position()
+        self._update_color()
+
+    def _update_position(self):
+        if not self._visible:
+            self._vertex_list.vertices = tuple([0] * ((len(self._coordinates) - 2) * 6))
+        elif self._rotation:
+            # Adjust all coordinates by the anchor.
+            anchor_x = self._anchor_x
+            anchor_y = self._anchor_y
+            coords = [[x - anchor_x, y - anchor_y] for x, y in self._coordinates]
+
+            # Rotate the polygon around its first vertex.
+            x, y = self._coordinates[0]
+            r = -math.radians(self._rotation)
+            cr = math.cos(r)
+            sr = math.sin(r)
+
+            for i, c in enumerate(coords):
+                c = [c[0] - x, c[1] - y]
+                c = [c[0] * cr - c[1] * sr + x, c[0] * sr + c[1] * cr + y]
+                coords[i] = c
+
+            # Triangulate the convex polygon.
+            triangles = []
+            for n in range(len(coords) - 2):
+                triangles += [coords[0], coords[n + 1], coords[n + 2]]
+
+            # Flattening the list before setting vertices to it.
+            self._vertex_list.vertices = tuple(value for coordinate in triangles for value in coordinate)
+
+        else:
+            # Adjust all coordinates by the anchor.
+            anchor_x = self._anchor_x
+            anchor_y = self._anchor_y
+            coords = [[x - anchor_x, y - anchor_y] for x, y in self._coordinates]
+
+            # Triangulate the convex polygon.
+            triangles = []
+            for n in range(len(coords) - 2):
+                triangles += [coords[0], coords[n + 1], coords[n + 2]]
+
+            # Flattening the list before setting vertices to it.
+            self._vertex_list.vertices = tuple(value for coordinate in triangles for value in coordinate)
+
+    def _update_color(self):
+        self._vertex_list.colors[:] = [*self._rgb, int(self._opacity)] * ((len(self._coordinates) - 2) * 3)
+
+    @property
+    def x(self):
+        """X coordinate of the shape.
+
+        :type: int or float
+        """
+        return self._coordinates[0][0]
+
+    @x.setter
+    def x(self, value):
+        self._coordinates[0][0] = value
+        self._update_position()
+
+    @property
+    def y(self):
+        """Y coordinate of the shape.
+
+        :type: int or float
+        """
+        return self._coordinates[0][1]
+
+    @y.setter
+    def y(self, value):
+        self._coordinates[0][1] = value
+        self._update_position()
+
+    @property
+    def position(self):
+        """The (x, y) coordinates of the shape, as a tuple.
+
+        :Parameters:
+            `x` : int or float
+                X coordinate of the shape.
+            `y` : int or float
+                Y coordinate of the shape.
+        """
+        return self._coordinates[0][0], self._coordinates[0][1]
+
+    @position.setter
+    def position(self, values):
+        self._coordinates[0][0], self._coordinates[0][1] = values
+        self._update_position()
+
+    @property
+    def rotation(self):
+        """Clockwise rotation of the polygon, in degrees.
+
+        The Polygon will be rotated about its (anchor_x, anchor_y)
+        position.
+
+        :type: float
+        """
+        return self._rotation
+
+    @rotation.setter
+    def rotation(self, rotation):
+        self._rotation = rotation
+        self._update_position()
+
+
+__all__ = ('Arc', 'Circle', 'Line', 'Rectangle', 'BorderedRectangle', 'Triangle', 'Star', 'Polygon', 'Sector')
